#!/bin/bash
#

# build_mongodb.sh <product_directory> <e10|e14|e15> <debug|prof> [tar] [source]

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e10|e14|e15> <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

get_arch()
{
    if [ ${OS1} = "Darwin" ]; then
      arch="osx"
    fi
    if [[ "${OS1}" == Linux ]]; then
      arch="linux"
    fi
}


# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=mongodb
cver=1.5.0
cppver=3.1.2
origpkgver=v3_4_6
pkgver=${origpkgver}a
ssibuildshims_version=v1_04_00
cppsourcetarname=mongo-cxx-driver-r${cppver}

get_this_dir

get_ssibuildshims

source define_basics --

get_arch

pkgdotver=`echo $origpkgver|sed -e 's/_/\./g'|sed -e 's/^v//g'`
pkgtarname=mongodb-${arch}-x86_64-${pkgdotver}
csourcetarname=mongo-c-driver-${cver}


if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# setup so we get the expected compiler
setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${PRODUCTS}
setup cmake v3_10_1
ups active

# Common configure args

common_config_args="--prefix=${pkgdir} --enable-shared"

if [ "${extraqual}" = "debug" ]; then
    cmake_build_type="Debug"
    extra_config_args="--enable-debug --enable-tracing"
elif [ "${extraqual}" = "prof" ]; then
    cmake_build_type="RelWithDebInfo"
    extra_config_args="--enable-debug-symbols=yes"
else
    cmake_build_type="Release"
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

cd ${tardir}

if [[ "${arch}" == "linux" ]]; then
 wget --no-check-certificate https://fastdl.mongodb.org/linux/mongodb-linux-x86_64-${pkgdotver}.tgz
 wget --no-check-certificate https://github.com/mongodb/mongo-c-driver/releases/download/${cver}/${csourcetarname}.tar.gz
elif [[ "${arch}" == "osx" ]];then
 curl --location --insecure --fail -O https://fastdl.mongodb.org/osx/mongodb-osx-x86_64-${pkgdotver}.tgz
 curl --location --insecure --fail -O https://github.com/mongodb/mongo-c-driver/releases/download/${cver}/${csourcetarname}.tar.gz
else
  echo "ERROR: I don't know how to build MongoDB UPS package on ${arch}!"
  exit 1
fi

set -x 

cd "${sourcedir}"
tar xf ${tardir}/${csourcetarname}.tar.gz || ssi_die "Unable to untar c source."
tar xf ${tardir}/${cppsourcetarname}.tar.gz || ssi_die "Unable to untar cpp source."
tar xf ${tardir}/${pkgtarname}.tgz || ssi_die "Unable to untar source."


# Install Binaries
mv ${pkgtarname}/bin ${pkgdir}

# Configure & Make C Driver
cd ${csourcetarname}
${sourcedir}/${csourcetarname}/configure ${common_config_args} ${extra_config_args} || ssi_die "ERROR: configure failed."
make -j ${ncore} || ssi_die "ERROR: c-driver make failed."
make install || ssi_die "ERROR: c-driver make install failed."
cd ..

# Configure & Make C++ Driver
cd ${cppsourcetarname}/build
PKG_CONFIG_PATH=${pkgdir}/lib/pkgconfig cmake -DBSONCXX_POLY_USE_MNMLSTC=1 -DCMAKE_BUILD_TYPE=${cmake_build_type} -DCMAKE_INSTALL_PREFIX=${pkgdir} ..
make -j ${ncore} || ssi_die "ERROR: cxx-driver make failed."
make install || ssi_die "ERROR: cxx-driver make install failed."
set +x

# Put includes in correct location
mkdir ${pkgdir}/../include
mv ${pkgdir}/include/bsoncxx/v*/bsoncxx ${pkgdir}/../include
mv ${pkgdir}/include/mongocxx/v*/mongocxx ${pkgdir}/../include
rm -rf ${pkgdir}/include/bsoncxx ${pkgdir}/include/mongocxx
mv ${pkgdir}/include/* ${pkgdir}/../include
rm -rf ${pkgdir}/include

# Install CMake Config Files
cp -a ${patchdir}/mongodb ${pkgdir}/lib
cp ${patchdir}/*.sh ${pkgdir}/bin

# Really don't want the build files sticking around
rm -rf ${sourcedir}

# Clean up.
cd "${pkgdir}"

# libtool and pkg-config files not useful. Also backups.
rm -rvf lib/pkgconfig lib/*.la
find . -name '*~' -exec rm -vf \{\} \;

${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "mongodb is installed at ${MONGODB_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
